#!/usr/bin/perl

# For option parsing
use Getopt::Long;

$VMTOOLS_INSTALL_LOG = "/var/log/vmware-tools-install.log";

#
# Asks the user a yes/no question
#
# @param Question for user
# @param Default value if ENTER is pressed with no value
#
# @return 1 for yes, 0 for no
sub user_confirm {
    my $question = shift @_;
    my $default = shift @_;

    while(1) {
        print "$question [$default]: ";
	select()->flush();

        my $answer = <STDIN>;
        chomp $answer;

        #if they didnt enter anything, use the default
        if($answer eq "") {
            $answer = $default;
        }

        # figure out if they entered yes or no
        if ($answer =~ /^(y|yes)$/i) {
            return 1;
        } elsif ($answer =~ /^(n|no)$/i) {
            return 0;
        } else {
            print "Please answer yes or no.\n";
        }
    }
}

#
# Print usage and exit
sub doUsage() {
    print "\nusage: firstboot-vmware-tools-config.pl [help|init|check|config]\n";
    print "Check and configure network.  No parameters for network check.\n";
    print "\t-help    Print this output and exit.\n";
    print "\t-init    Force initial install of VMwareTools.\n";
    print "\t-config  Start VMware Tools configuration from user input.\n";
    print "\t-check   Check VMware Tools configuration.\n";
    print "\t         (Configure if errors detected.)\n";
    exit(99);
}

#
# Parse arguments and return operation mode or fail.
# check - Perform check of setup and prompt user for action if an error is detected.
# config - Prompt user for configuration.
# init - Attempt to read initial config from OVF and prompt user if problem is detected.
#
# @param - none
# @return mode string: 'init', 'check' or 'config'
sub processArguments {
    my $result = "check"; # Default mode 

    # Local variables to hold the arg values
    my ($help, $init, $check, $config);

    # Parse command line and get values
    GetOptions('help'       => \$help,
               'init'       => \$init,
               'check'      => \$check,
               'config'     => \$config
    );
    
    # Check in order to give more involved arguments precidence.
    my $num_args = $#ARGV + 1;
    if ($num_args > 0 || $help) {
        doUsage;
    }
    elsif ($config) {
        $result = "config";
    }
    elsif ($init) {
        $result = "init";
    } else {
        return $result;
    }

    return $result;
}


#
# Install vmware tools
#
# @returns 0 for success and non-zero for failure
sub install_vmware_tools {
    my ($rc) = 0;
    open(LOGFILE, ">>", $VMTOOLS_INSTALL_LOG) or die "Unable to open log file:", $VMTOOLS_INSTALL_LOG, "\n";
    print LOGFILE "Installing VMware Tools ...\n";

    # First ensure there is nothing mounted on /cdrom
    # Loop is necessary to handle multiple mounts
    system("df | grep \/cdrom");
    until ( $? != 0 ) {
        sleep(1);
        system("umount /cdrom &> /dev/null");
        system("df | grep \/cdrom");
    }

    print "\n\nBefore you can continue the ONTAP tools virtual appliance \n";
    print "installation, you must install the VMware Tools:\n\n";
    print "   1.  Select VM > Guest OS > Install VMware Tools.\n\n";
    print "                     OR\n\n";
    print "       Click on \"Install VMware Tools\" pop-up box on the vSphere Web Client.\n";
    print "\n";
    print "   2.  Follow the prompts provided by the VMware Tools wizard.\n\n";
    print "Once you click on mount, the installation process will automatically continue.";

    # You want to have a blind umount to clear the flakey /cdrom device
    system("umount /cdrom &> /dev/null");
    $canmount = -1;
    while ($canmount != 0) {
        system("mount -o ro /cdrom 1>>$VMTOOLS_INSTALL_LOG 2>&1");
        $canmount = $?;
        if ( $canmount != 0 ) {
            print LOGFILE "ISO mount failed.\n\nMake sure that a valid ISO is mounted on the virtual CD/DVD device.\n";
            $canmount = 1;
        }
        sleep 5;
    }
    print LOGFILE "Successfully mounted ISO..\n";
    print "\nSuccessfully mounted ISO..\n";
    
    print LOGFILE "Searching for VMware Tools archive...\n";
    print "\nSearching for VMware Tools archive...\n";
    open(FIND, "find /cdrom -maxdepth 1 -type f -name 'VMwareTools-*.tar.gz' -print |") or die "Failed to execute find";
    $rc = 99;
    while (<FIND>) {
        # Look for the link status
        if ($_ =~ m/VMwareTools/) {
            print LOGFILE "Found VMwareTools archive: $_\n";
            $rc = 0;
            break;
        }
    }
    close(FIND);
    if ( $rc != 0 ) {
        print LOGFILE "VMware Tools ISO mount failed.\n\n";
        print "VMware Tools ISO mount failed.\n\n";
        print LOGFILE "Make sure that a valid VMware Tools ISO is mounted on the virtual CD/DVD device.\n";
        print "Make sure that a valid VMware Tools ISO is mounted on\n";
        print "the virtual CD/DVD device.\n";
	    $rc = 1;
    }

    if ( $rc == 0 ) {
        # Extract the VMware Tools tarball into the tmp directory
        # The cat command is used because tar does not accept wild cards in the file name parameter
        print "Unpacking VMware Tools archive...\n";
        print LOGFILE "Unpacking VMware Tools archive...\n";
        system("tar xzf /cdrom/VMwareTools-*.tar.gz -C /tmp 1>>$VMTOOLS_INSTALL_LOG 2>&1");
        if ($? != 0) {
            print "VMware Tools unpack failed\n";
            $rc=1;
        } else {
            # Install the 4.0.0 version of tools
            print "Running VMware Tools install script... This may take several minutes...\n";
            print LOGFILE "Running VMware Tools install script..\n";
            system("echo 'yes' | /tmp/vmware-tools-distrib/vmware-install.pl >> $VMTOOLS_INSTALL_LOG 2>&1");
            if ($? != 0) {
                print "VMware Tools installer failed\n";
                $rc=2;
            }
        }
    }

    if ($rc == 0) {
        print "Extracted tar and Installed VMware tools ...\n";
        $VMWARE_CONFIG_CHANGED=1;
    }
    
    # Cleanup the tmp directory
    system("rm -rf /tmp/vmware-tools-distrib");
    # Ensure there is no install media attached to the guest
    system("umount /cdrom &> /dev/null");
    system("eject /cdrom &> /dev/null");
    
    close(LOGFILE);
    return $rc;
}

#
# Upgrade vmware tools
#
# @returns 0 for success and non-zero for failure
sub upgrade_vmware_tools {
    my ($rc) = 0;
    # Upgrade to the latest version of tools available
    open(LOGFILE, ">>", $VMTOOLS_INSTALL_LOG) or die "Unable to open log file:", $VMTOOLS_INSTALL_LOG, "\n";
    print LOGFILE "VMware tools upgrade with Non default options ...\n";
    print "VMware tools upgrade with Non default options ...\n";
    system("/usr/sbin/vmware-tools-upgrader -p --default --clobber-kernel-modules=pvscsi,vmxnet3 1>>$VMTOOLS_INSTALL_LOG 2>&1");

    # Search the log file for a Usage error message from the upgrader script. This indicates the --clobber-kernel-modules option is
    # not valid for the version of tools that's being upgraded to (the option was introduced in VMware 4.1.0). This search is 
    # necessary because the vmware-tools-upgrader program does not give a bad error code when the installation command fails.
    system("grep \'Usage.*vmware-install.pl\' $VMTOOLS_INSTALL_LOG &> /dev/null");
    if ($? == 0) {
        # If the --clobber-kernel-modules option wasn't valid, retry the upgrade with just the default options
        print LOGFILE "Retrying upgrade without --clobber-kernel-modules option ...\n";
        open(LOGFILE, ">>", $VMTOOLS_INSTALL_LOG) or die "Unable to open log file:", $VMTOOLS_INSTALL_LOG, "\n";
        print "Retrying upgrade without --clobber-kernel-modules option ...\n";
        system("/usr/sbin/vmware-tools-upgrader -p --default 1>> $VMTOOLS_INSTALL_LOG 2>&1");
        if ($? != 0) {
            print "VMware Tools upgrade failed\n";
            $rc=1;
        } else {
            print "VMware tools upgrade with default options ...\n";
        }
    }
    
    # Cleanup the tmp directory
    system("rm -rf /tmp/vmware-tools-distrib");
    
    close(LOGFILE);
    $VMWARE_CONFIG_CHANGED=1;
    return $rc;
}

sub restart_networking() {
    open(LOGFILE, ">>", $VMTOOLS_INSTALL_LOG) or die "Unable to open log file:", $VMTOOLS_INSTALL_LOG, "\n";
    print LOGFILE "Restarting networking with new drivers ...\n";
    print "Restarting networking with new drivers ...\n";

    # Restart networking to use the new net driver 
    system("/etc/init.d/networking stop &>> $VMTOOLS_INSTALL_LOG");
    system("/sbin/rmmod pcnet32 &>> $VMTOOLS_INSTALL_LOG");
    system("/sbin/rmmod vmxnet &>> $VMTOOLS_INSTALL_LOG");
    system("/sbin/modprobe vmxnet &>> $VMTOOLS_INSTALL_LOG");
    # If the loading of the vmxnet module failed, re-introduce the pcnet32 module so networking won't fail
    if ($? != 0) {
        system("/sbin/modprobe pcnet32 &>> $VMTOOLS_INSTALL_LOG");
    }

    system("/etc/init.d/networking start &>> $VMTOOLS_INSTALL_LOG");
    close(LOGFILE);
}

#Check other script files where similar function is added as that also needs modification
sub get_vmtoolsd_path() {
	if (-e "/usr/bin/vmtoolsd") {
		return "/usr/bin/vmtoolsd";
	}
	elsif (-e "/usr/sbin/vmtoolsd") {
		return "/usr/sbin/vmtoolsd";
	}
	else {
		return "error";
	}
}

#
# Check VMwareTools
#
# @returns 0 for OK or non-zero for problem
sub check_vmware_tools() {
    #print "Checking VMware Tools Install ...\n";
    my ($rc)=0;
    
    my $vmtoosdpath = get_vmtoolsd_path();
    if ($vmtoosdpath eq "error") {
        print "ERROR: vmtoolsd not found.\n";
        $rc=1;
    }
    
    # Network driver check
    system("lsmod | grep -q vmxnet &> /dev/null");
    if ($? != 0) {
        print "ERROR: VMware Tools network module not detected\n";
        $rc=2;
    }
    
    # Balloon memory driver
    system("lsmod | grep -q vmmemctl &> /dev/null");
    if ($? != 0) {
        print "ERROR: VMware Tools memory module not detected\n";
        $rc=3;
    }
    
    # Filesystem module
    system("lsmod | grep -q vmhgfs &> /dev/null");
    if ($? != 0) {
        print "ERROR: VMware Tools filesystem module not detected\n";
        $rc=4;
    }
    
    return $rc;
}

##################################################################################################################
# Main execution
##################################################################################################################
# At boot, the appliance's startup messages are supressed by redirecting them to an alternate console /dev/ttyS0
# For the interactive portions of this program, we'll need to explicitly switch STDIN and STDOUT to use the real console at /dev/tty1
sub main() {
    open(STDIN, "</dev/tty1");
    open(STDOUT, ">/dev/tty1");
    
    my ($mode) = processArguments();
    my ($rc) = 0;
    
    # Clear the screen
    #print (((" " x 80) . "\n") x 25);
    system("clear");

    if ($mode eq "check") {
        # We don't do anything for -check
        # right now.
        print "\nVMware Tools Check\n";
    } elsif ($mode eq "init") {
    #print "\nVMware Tools installation\n";
        # This handles initial configuration (aka. -init)
        # Ask them which method they would like to use
        #
        # First, install VMware Tools
        #if (install_vmware_tools() == 0) {
        #}
    } # End init
    elsif ($mode eq "config") {
        print "\nVMware Tools configuration\n";
        # Try to upgrade
        # Check if VMware Tools is not already installed
        #install_vmware_tools();
    } # End config
    
    if ($VMWARE_CONFIG_CHANGED) {
        #restart_networking();
    } else {
        # Start check
        # Getting here means that connection is ok. Now do the initial check of the network validity and exit immediately 
        # if the the network is valid.
        $valid = check_vmware_tools();
        if ($valid == 0) {
            print "VMWare Tools appears OK\n";
        } else {
            print "VMware Tools check failed\n";
            if (!user_confirm("Would you like to re-install VMware Tools?", 'y')) {
                print "\n";
                print "You have chosen to enable the virtual appliance with a possible\n";
                print "VMware Tools configuration error. Please resolve outstanding VMware Tools\n";
                print "issues and restart this appliance.\n";
                print "\n";
                print "Press ENTER to continue.";
		select()->flush();
                $continue = <STDIN>;
                $rc = 1; # Fail
            } else {
                #$rc = install_vmware_tools();
            }
        }
    }
    
    if ($VMWARE_CONFIG_CHANGED) {
        print "\n";
        print "The operating system drivers have been modified.\n";
        print "A reboot is required for proper operation.\n";
        print "\n";
        print "IMPORTANT: You must now check the appliance CD/DVD configuration:\n\n";
        print "   1.  Select VM > Edit Settings.\n\n";
        print "   2.  Set CD/DVD device type to: Client Device.\n\n";
        
        #print "Now press ENTER to reboot.";
        #$continue = <STDIN>;
        sleep(10);
        print "\n";
        print "Rebooting ...";
        sleep(5);

        $rc = 66; # reboot
    }

    exit($rc);
}

main;

