#!/bin/bash

# $Id$
# Copyright (c) 2013 Network Appliance, Inc.
# All rights reserved.


##########################################################################################################
##########################################################################################################
# This script will upgrade vpserver and any other packages shipped with udpate ISO
# Workflow: Client has mounted iso already. This script will stop
# services, wait, upgrade, start vpserver, and wait.
##########################################################################################################
##########################################################################################################
. /lib/lsb/init-functions

PKGNAME=vmware-tools

MAINT_USER=maint
MAINT_GROUP=maintenance
MAINT_HOME="/home/${MAINT_USER}"
MAINT_SCRIPTS="${MAINT_HOME}/scripts"
LOG_DIR="/var/log"
VMTOOLS_INSTALL_LOG="${LOG_DIR}/vmware-tools-install.log";
PLATFORM_SETUP_SCRIPTS="${MAINT_HOME}/setup_scripts"
MAINT_APP_RC_DIR="${MAINT_HOME}/app.d"

. ${MAINT_SCRIPTS}/commonfunctions.sh

#
# Variable definitions
#
timestamp=`date +%Y%m%d%H%M%S`

VMWARE_CONFIG_CHANGED=0

# Make sure that we trap any control characters so that the user
# does not have access to the shell.
trap logOut 1 2 3 4 5 6 9 11 15


function echo_log()
{
    tstamp=`date +'%b %d %T'`
    echo "$@"
    echo "${tstamp}:  $@" >> ${VMTOOLS_INSTALL_LOG}
}

function log()
{
    tstamp=`date +'%b %d %T'`
    echo "${tstamp}:  $@" &>> ${VMTOOLS_INSTALL_LOG}
}

function ejectCD() {
    # Attempt to clean up any CD mounts
    umount /cdrom &> /dev/null
    eject /dev/cdrom &> /dev/null
}

#Message to display when exiting with an error.
function error()
{
    echo
    read -n1 -p "Press ENTER to continue." anykey
    echo
    ejectCD
    logOut
}

#
#Add iso in cdrom to sources
#
function sourceISO()
{
    # First ensure there is nothing mounted on /cdrom
    # Loop is necessary to handle multiple mounts
    grepResult=`df | grep \/cdrom`
    until [ -z "${grepResult}" ] ; do
        sleep 1
        umount /cdrom &> /dev/null
        grepResult=`df | grep \/cdrom`
    done
    
    # You want to have a blind umount to clear the flakey /cdrom device
    umount /cdrom &> /dev/null
    mount /cdrom &>> ${VMTOOLS_INSTALL_LOG}
    if [[ $? != 0 ]] ; then
        echo_log "ISO mount failed."
        echo
        echo_log "Make sure that a valid ISO is mounted on the virtual CD/DVD device."
	    error
    fi
    
    if [[ ! -f $(find /cdrom -maxdepth 1 -type f -name 'VMwareTools-*.tar.gz' -print) ]] ; then
        echo_log "VMware Tools ISO mount failed."
        echo
        echo_log "Make sure that a valid VMware Tools ISO is mounted on the virtual CD/DVD device."
	    error
    fi
    
}

#
#Stop services and wait
#
# Perform the VMware Tools Install/upgrade
#
function install_vmware_tools()
{
    local rc=0
    
    # Install/Upgrade VMware Tools
    echo_log "Installing VMware Tools..."
    # Extract the VMware Tools tarball into the tmp directory
    # The cat command is used because tar does not accept wild cards in the file name parameter
    echo_log "Unpacking VMware Tools..."
    tar xzf /cdrom/VMwareTools-*.tar.gz -C /tmp 2>&1 >> ${VMTOOLS_INSTALL_LOG}
    if [[ $? != 0 ]] ; then
        echo_log "VMware Tools unpack failed"
        rc=1
    else
        # Install VMware Tools
        echo_log "Running VMware Tools installer..."
        /tmp/vmware-tools-distrib/vmware-install.pl -default 2>&1 >> ${VMTOOLS_INSTALL_LOG}
        if [[ $? != 0 ]] ; then
            echo_log "VMware Tools installer failed"
            rc=2
        fi
    fi

    if [[ $rc == 0 ]] ; then
        echo_log "Extracted tar and Installed VMware tools ..."
        VMWARE_CONFIG_CHANGED=1;
    fi
    
    # Cleanup the tmp directory
    rm -rf /tmp/vmware-tools-distrib

    if [[ $rc -eq 0 ]]; then
        echo
        echo_log "Successfully installed VMware Tools"
        echo
        upgradestatus=1
        ejectCD
    else
        echo
        echo_log "VMware Tools install failed."
        echo_log "Contact NetApp customer support and provide the necessary log information."
        echo_log "Log information can be fetched from the following locations:"
        echo_log "* ${VMTOOLS_INSTALL_LOG}"
        error
    fi
}

# Currently not used
function upgrade_vmware_tools() {
    # Upgrade to the latest version of tools available
    echo_log "VMware tools upgrade with Non default options ..."
    /usr/sbin/vmware-tools-upgrader -p --default --clobber-kernel-modules=pvscsi,vmxnet3 >> ${VMTOOLS_INSTALL_LOG}

    # Search the log file for a Usage error message from the upgrader script. This indicates the --clobber-kernel-modules option is
    # not valid for the version of tools that's being upgraded to (the option was introduced in VMware 4.1.0). This search is 
    # necessary because the vmware-tools-upgrader program does not give a bad error code when the installation command fails.
    grep \'Usage.*vmware-install.pl\' ${VMTOOLS_INSTALL_LOG} &> /dev/null
    if [[ $? == 0 ]] ; then
        # If the --clobber-kernel-modules option wasn't valid, retry the upgrade with just the default options
        echo_log "Retrying upgrade without --clobber-kernel-modules option ..."
        /usr/sbin/vmware-tools-upgrader -p --default >> ${VMTOOLS_INSTALL_LOG}
        if [[ $? != 0 ]] ; then
            echo_log "VMware Tools upgrade failed"
            error
        else
            echo_log "VMware tools upgrade with default options ..."
        fi
    fi
    
    # Cleanup the tmp directory
    rm -rf /tmp/vmware-tools-distrib
    
    VMWARE_CONFIG_CHANGED=1
}

function restart_networking() {
    echo_log "Restarting networking with new drivers ..."

    # Restart networking to use the new net driver 
    /etc/init.d/networking stop >> ${VMTOOLS_INSTALL_LOG}
    /sbin/rmmod pcnet32 >> ${VMTOOLS_INSTALL_LOG}
    /sbin/rmmod vmxnet >> ${VMTOOLS_INSTALL_LOG}
    /sbin/modprobe vmxnet >> ${VMTOOLS_INSTALL_LOG}
    # If the loading of the vmxnet module failed, re-introduce the pcnet32 module so networking won't fail
    if [[ $? != 0 ]] ; then
        /sbin/modprobe pcnet32 >> ${VMTOOLS_INSTALL_LOG}
    fi

    /etc/init.d/networking start >> ${VMTOOLS_INSTALL_LOG}
}

#
# Check VMwareTools
#
# @returns 0 for OK or non-zero for problem
function check_vmware_tools() {
    echo_log "Checking VMware Tools Install ..."
    
    if [[ ! -e "/usr/bin/vmtoolsd" ]] && [[ ! -e "/usr/sbin/vmtoolsd" ]] ; then
        echo_log "ERROR: vmware-tools-upgrader not found."
        error
    fi
    
    # Network driver check
    lsmod | grep -q vmxnet &> /dev/null
    if [[ $? != 0 ]] ; then
        echo_log "ERROR: VMware Tools network module not detected"
        error
    fi
    
    # Balloon memory driver
    lsmod | grep -q vmmemctl &> /dev/null
    if [[ $? != 0 ]] ; then
        echo_log "ERROR: VMware Tools memory module not detected"
        error
    fi
    
    # Filesystem module
    lsmod | grep -q vmhgfs &> /dev/null
    if [[ $? != 0 ]] ; then
        echo_log "ERROR: VMware Tools filesystem module not detected"
        error
    fi
}

# ************ Main section ************

function main() {
    #
    #Warn user that this process will bring down the services
    #
    echo
    echo "Install/Upgrade VMware Tools"
    echo "This process will take applications offline and"
    echo  "require a system reboot."
    echo
    echo "You should make a backup of this virtual appliance"
    echo "before you install or upgrade VMware Tools."
    echo
    while [[ 1 ]]
    do
        read -r -p "Do you wish to continue? (y/n):" answer
        if [ "$answer" == "n" ]  || [ "$answer" == "N" ]; then
            echo
            echo_log "VMware Tools install cancelled."
            echo
            logOut
        fi
        if [ "$answer" == "y" ]  || [ "$answer" == "Y" ]; then
            break
        fi
    done

    sourceISO

    # @todo use a more generic application start/stop
    serverStartStop vpserver stop

    # For now, we are just going to force install if run from the menu.
    install_vmware_tools
    sleep 5

    # @todo use a more generic application start/stop
    serverStartStop vpserver start

    # Make sure that on successfull upgrade, logout from the maintenance console.
    # Return code 100 make sure that session logsout from maintenance console.
    # Any return code other than 100 is failure and maintenance console goes to main menu in case of failure.
    #if [[ ${servicestart} -eq 1 && ${upgradestatus} -eq 1 ]]; then
    if [[ ${upgradestatus} -eq 1 ]]; then
        echo
        touch /var/lib/vplatform/vmware-tools-config-CONFIGURED &> /dev/null
        echo
        echo "IMPORTANT: You must now check the appliance CD/DVD configuration:"
        echo
        echo "   1.  Select VM > Edit Settings."
        echo
        echo "   2.  Set CD/DVD device type to: Client Device."
        echo
        exit 100
    fi
    logOut
}

main "$@"
